local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI then return end

local pairs = pairs
local CreateFrame, UIParent = CreateFrame, UIParent

AceGUI:RegisterLayout("Relative",
	function(content, children)
		local rows
		if #children % content.WidthSplit ~= 0 then
			rows = (#children + content.WidthSplit - #children % content.WidthSplit) / content.WidthSplit
		else
			rows = #children / content.WidthSplit
		end
		local w = content:GetWidth() / content.WidthSplit - content.padding / content.WidthSplit
		local h = content:GetHeight() / rows - content.padding / rows
		
		for i, child in ipairs(children) do		
			child.frame:Show()
			child.frame:ClearAllPoints()			
			if i == 1 then
				child.frame:SetPoint("TOPLEFT", content, content.padding, -content.padding)								
			elseif i % content.WidthSplit == 1 or content.WidthSplit == 1 then
				child.frame:SetPoint("TOPLEFT", children[i - content.WidthSplit].frame, "BOTTOMLEFT", 0, -content.padding)
			else
				child.frame:SetPoint("TOPLEFT", children[i - 1].frame, "TOPRIGHT", content.padding, 0)
			end
			
			child.frame:SetWidth(w - content.padding)
			child.frame:SetHeight(h - content.padding)
		end	
		
		if content.obj.LayoutFinished then
			content.obj.LayoutFinished(content.obj, content:GetWidth(), content:GetHeight())
		end
	end)

local Type, Version = "UIF-GridSlectFrame", 1
if (AceGUI:GetWidgetVersion(Type) or 0) < Version then
	local NormalColor = {0.16, 0.16, 0.16, 1}
	local HoverColor = {0.23, 0.23, 0.23, 1}
	local SelctedColor = {0.4, 0.4, 0.16, 1}

	--[[-----------------------------------------------------------------------------
	Scripts
	-------------------------------------------------------------------------------]]
	local function Frame_OnEnter(this)
		local self = this.obj
		self:SetTexture(unpack(HoverColor))
		self:Fire("OnEnter")
	end
	
	local function Frame_OnLeave(this)
		local self = this.obj
		if self.checked then
			self:SetTexture(unpack(SelctedColor))
		else
			self:SetTexture(unpack(NormalColor))
		end
		self:Fire("OnLeave")
	end
	
	local function Frame_OnMouseDown(this)
		local self = this.obj
		self.checked = true
		self:SetTexture(unpack(SelctedColor))
		self:Fire("OnValueSelected", self.value)
	end	
	
	--[[-----------------------------------------------------------------------------
	Methods
	-------------------------------------------------------------------------------]]
	local methods = {
		["OnAcquire"] = function(self)
			self:SetWidth(50)
			self:SetHeight(50)
		end,
		
		["SetText"] = function(self, text)
			self.text:SetText(text)
		end,
		
		["SetTexture"] = function(self, r, g, b, a)
			self.texture:SetTexture(r, g, b, a or 1)
		end,
		
		["SetValue"] = function(self, value)
			self.value = value
		end,
		
		["GetValue"] = function(self)
			return self.value
		end,
		
		["SetSelected"] = function(self, selected)
			if selected then
				self.checked = true
				self:SetTexture(unpack(SelctedColor))
			else
				self.checked = false
				self:SetTexture(unpack(NormalColor))
			end
		end,
	}
	
	--[[-----------------------------------------------------------------------------
	Constructor
	-------------------------------------------------------------------------------]]
	local function Constructor()
		local count = AceGUI:GetNextWidgetNum(Type)
		local frame = CreateFrame("Frame", Type..count, UIParent)
		
		frame:SetScript("OnEnter", Frame_OnEnter)
		frame:SetScript("OnLeave", Frame_OnLeave)
		frame:SetScript("OnMouseDown", Frame_OnMouseDown)
		
		local text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")		
		text:SetAllPoints()
		local texture = frame:CreateTexture(nil, "ARTOWRK")
		texture:SetAllPoints()
		texture:SetTexture(unpack(NormalColor))
		
		local widget = {
			frame = frame,
			texture = texture,
			text = text,
			type = Type,
		}
		
		for method, func in pairs(methods) do
			widget[method] = func
		end
		
		for i, v in pairs({frame}) do
			v.obj = widget
		end
		
		return AceGUI:RegisterAsWidget(widget)
	end
	AceGUI:RegisterWidgetType(Type, Constructor, Version)
end

local Type, Version = "UIF-GridSelect", 1
if (AceGUI:GetWidgetVersion(Type) or 0) < Version then	
	DefinedTypes = {
		Anchor = {
			{value = "TOPLEFT", text = "Top Left"},
			{value = "TOP", text = "Top"},
			{value = "TOPRIGHT", text = "Top Right"},
			{value = "LEFT", text = "Left"},
			{value = "CENTER", text = "Center"},
			{value = "RIGHT", text = "Right"},
			{value = "BOTTOMLEFT", text = "Bottom Left"},
			{value = "BOTTOM", text = "Bottom"},
			{value = "BOTTOMRIGHT", text = "Bottom Right"},
		}
	}

	--[[-----------------------------------------------------------------------------
	Callbacks
	-------------------------------------------------------------------------------]]
	local function OnValueSelected(this, event, value)
		local self = this:GetUserData("parent")
		self:SetValue(value)		
		self:Fire("OnValueChanged", value)
	end
	--[[-----------------------------------------------------------------------------
	Scripts
	-------------------------------------------------------------------------------]]

	--[[-----------------------------------------------------------------------------
	Methods
	-------------------------------------------------------------------------------]]
	local methods = {
		["OnAcquire"] = function(self)
			self.content.WidthSplit = 2
			self.content.padding = 2
			
			self:SetWidth(250)
			self:SetHeight(150)		
		end,
		
		["OnRelease"] = function(self)
			self.list = nil
			wipe(self.items)
		end,
		
		["SetLabel"] = function(self, text)
			self.label:SetText(text)
		end,
		
		["SetPadding"] = function(self, padding)
			self.content.padding = padding
			self:DoLayout()
		end,
		
		["SetColumnCount"] = function(self, count)
			assert(count > 0)
			self.content.WidthSplit = count
			self:DoLayout()
		end,
		
		["SetList"] = function(self, list)
			self:ReleaseChildren()
			self.list = list
			
			for i, v in pairs(list) do
				local f = AceGUI:Create("UIF-GridSlectFrame")
				f:SetValue(i)
				f:SetText(v)
				f:SetUserData("parent", self)
				f:SetCallback("OnValueSelected", OnValueSelected)
				self:AddChild(f)
				tinsert(self.items, f)
			end
		end,
		
		["SetSortedList"] = function(self, list)
			self:ReleaseChildren()
			self.list = list
			
			for i, v in pairs(list) do
				local f = AceGUI:Create("UIF-GridSlectFrame")
				f:SetValue(v.value)
				f:SetText(v.text)
				f:SetUserData("parent", self)
				f:SetCallback("OnValueSelected", OnValueSelected)
				self:AddChild(f)
				tinsert(self.items, f)
			end
		end,
		
		["SetValue"] = function(self, value)
			for i, item in pairs(self.items) do
				if item:GetValue() ~= value then
					item:SetSelected()
				else
					item:SetSelected(true)
				end
			end
		end,
		
		["SetType"] = function(self, type)
			if DefinedTypes[type] then
				self:ReleaseChildren()
				self:SetSortedList(DefinedTypes[type])
			end
		end
	}
	
	--[[-----------------------------------------------------------------------------
	Constructor
	-------------------------------------------------------------------------------]]
	local Backdrop  = {
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = GetUIFPath().."\\textures\\border3.tga",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	}

	local function Constructor()
		local count = AceGUI:GetNextWidgetNum(Type)
		local frame = CreateFrame("Frame", Type..count, UIParent)
		
		-- Label
		local label = frame:CreateFontString(Type.."Label"..count, "OVERLAY", "GameFontNormal")
		label:SetPoint("TOPLEFT", 1, -1)
		
		-- Pane
		local pane = CreateFrame("Frame", Type.."Pane"..count, frame)
		pane:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 0, -1)
		pane:SetPoint("BOTTOMRIGHT", -1, 1)
		pane:EnableMouse(true)
		pane:SetBackdrop(Backdrop)
		pane:SetBackdropColor(0.1, 0.1, 0.1, 1)
		pane:SetBackdropBorderColor(0.4, 0.4, 0.4)	
		
		local content = CreateFrame("Frame", Type.."Content"..count, pane)
		content:SetPoint("TOPLEFT", 2, -2)
		content:SetPoint("BOTTOMRIGHT", -2, 2)
		
		local widget = {
			items = {},
			frame = frame,
			content = content,
			label = label,
			type = Type,
		}
		
		for method, func in pairs(methods) do
			widget[method] = func
		end
		
		for i, v in pairs({frame}) do
			v.obj = widget
		end
		
		return AceGUI:RegisterAsContainer(widget)
	end
	AceGUI:RegisterWidgetType(Type, Constructor, Version)
end